@@ -1,7 +1,7 @@
 /*
  * ======= General internationalization, for Lisp programs too =======
  * Copyright (C) 1990-2005 Bruno Haible
- * Copyright (C) 1998-2006 Sam Steingold
+ * Copyright (C) 1998-2008 Sam Steingold
  * GPL2
  */
 
@@ -182,17 +182,18 @@ DEFUN(I18N:TEXTDOMAINDIR, domain)
 
 DEFUN(I18N:SET-TEXTDOMAINDIR, domain directory)
 { /* sets the message catalog directory for the given domain. */
-  object domain = (STACK_1=check_string(STACK_1));
+  STACK_1 = check_string(STACK_1);
  #ifdef GNU_GETTEXT
-  /* Check and use default directory, because the bindtextdomain()
-     documentation recommends that the argument be an absolute pathname,
-     to protect against later chdir() calls. */
-  object directory = pathname_to_OSdir(STACK_0,true);
-  with_string_0(STACK_1/*domain*/,Symbol_value(S(ascii)),domain_asciz, {
-    begin_system_call();
-    bindtextdomain(domain_asciz,TheAsciz(directory));
-    end_system_call();
-  });
+  { /* Check and use default directory, because the bindtextdomain()
+       documentation recommends that the argument be an absolute pathname,
+       to protect against later chdir() calls. */
+    object directory = pathname_to_OSdir(STACK_0,true);
+    with_string_0(STACK_1/*domain*/,Symbol_value(S(ascii)),domain_asciz, {
+        begin_system_call();
+        bindtextdomain(domain_asciz,TheAsciz(directory));
+        end_system_call();
+      });
+  }
  #endif
   VALUES1(STACK_0);
   skipSTACK(2);
@@ -260,7 +261,7 @@ static void get_locale_info (int what, char**res, int *res_size) {
   end_system_call();
   if (val == 0) OS_error();
   if (val > *res_size) {
-    *res = (char*)my_realloc(*res,val);
+    *res = (char*)clisp_realloc(*res,val);
     if (*res == NULL) OS_error();
     *res_size = val;
     begin_system_call();
@@ -385,7 +386,7 @@ static void locale_bool_to_STACK (int what, char**res, int* res_size) {
 DEFUN(I18N:LOCALE-CONV,)
 { /* call GetLocaleInfo(3) */
   int res_size = GET_LOCALE_INFO_BUF_SIZE;
-  char *res = my_malloc(res_size);
+  char *res = clisp_malloc(res_size);
   locale_string_to_STACK(LOCALE_SDECIMAL,&res,&res_size);
   locale_string_to_STACK(LOCALE_STHOUSAND,&res,&res_size);
   thousands_sep_to_STACK(LOCALE_SGROUPING,&res,&res_size);
@@ -480,7 +481,7 @@ DEFCHECKER(check_nl_item,CODESET                                        \
 #elif defined(WIN32_NATIVE)
 # define get_lang_info(what)  get_locale_info(what,&res,&res_size)
 # define res_to_obj() (asciz_to_string(res,GLO(misc_encoding)))
-# define DECLARE_RES  int res_size=GET_LOCALE_INFO_BUF_SIZE; char *res=(char*)my_malloc(res_size)
+# define DECLARE_RES  int res_size=GET_LOCALE_INFO_BUF_SIZE; char *res=(char*)clisp_malloc(res_size)
 # define FINISH_RES   begin_system_call(); free(res); end_system_call()
 #endif
 DEFUNR(I18N:LANGUAGE-INFORMATION,&optional item)
