@@ -1,7 +1,7 @@
 ;; CLISP interface to PARI <http://pari.math.u-bordeaux.fr/>
 ;; Copyright (C) 1995 Michael Stoll
-;; Copyright (C) 2004-2007 Sam Steingold
-;; This is free software, distributed under the GNU GPL
+;; Copyright (C) 2004-2008 Sam Steingold
+;; This is free software, distributed under the GNU GPL (v2+)
 
 (defpackage "PARI"
   (:modern t)
@@ -203,7 +203,7 @@
 ;;; /* init.c */
 
 ;; long allocatemoremem(size_t newsize);
-(def-call-out allocatemoremem (:arguments (newsize ulong)) (:return-type long))
+(def-call-out allocatemoremem (:arguments (newsize size_t)) (:return-type long))
 (defun allocatemem (&optional (newsize 0)) (allocatemoremem newsize))
 
 (c-lines "#include \"cpari.h\"~%")
