@@ -13,6 +13,7 @@
    #+(or :win32 :cygwin) #:file-properties #+unix #:make-xterm-io-stream
    #:priority #:openlog #:setlogmask #:syslog #:closelog #:process-id #:getppid
    #:getsid #:setsid #:getpgrp #:setpgrp #:setreuid #:setregid #:kill #:sync
+   #:errno #:strerror #:hostid #:domainname #:file-size #:user-shells
    #:erf #:erfc #:j0 #:j1 #:jn #:y0 #:y1 #:yn #:tgamma #:lgamma #:ffs))
 
 (pushnew :syscalls *features*)
@@ -29,13 +30,13 @@
 )
 ;;; ============================================================
 #+unix (progn
-(export '(getuid getgid geteuid getegid getpgid getgroups))
-(defsetf getuid posix::%setuid)
-(defsetf getgid posix::%setgid)
-(defsetf geteuid posix::%seteuid)
-(defsetf getegid posix::%setegid)
-(defsetf getpgid posix::%setpgid)
-(defsetf getgroups posix::%setgroups)
+(export '(uid gid euid egid pgid groups))
+(defsetf uid %setuid)
+(defsetf gid %setgid)
+(defsetf euid %seteuid)
+(defsetf egid %setegid)
+(defsetf pgid %setpgid)
+(defsetf groups %setgroups)
 )
 ;;; ============================================================
 (defmacro with-stream-lock ((stream &rest options) &body body)
@@ -43,6 +44,10 @@
   `(unwind-protect (progn (stream-lock ,stream t ,@options) ,@body)
      (stream-lock ,stream nil ,@options)))
 ;;; ============================================================
+(defsetf file-size %set-file-size)
+(defsetf hostid %sethostid)
+(defsetf domainname %setdomainname)
+;;; ============================================================
 (defun syslog (severity facility format &rest args)
   (%syslog severity facility (apply #'format nil format args)))
 ;; A compiler macro here is better than a simple (declaim (inline syslog))
@@ -268,7 +273,9 @@
                    (* page-size (sysconf :AVPHYS-PAGES))))
   #+win32 (let ((mem-stat (memory-status)))
             (values (memstat-total-physical mem-stat)
-                    (memstat-avail-physical mem-stat))))
+                    (memstat-avail-physical mem-stat)))
+  #-(or unix win32)
+  (error "~S: only ~S and ~S are supported" 'physical-memory :unix :win32))
 
 ;;;--------------------------------------------------------------------------
 ;;; service / port map
@@ -283,7 +290,9 @@
 (defvar *services*)
 (defvar *services-file*
   #+unix "/etc/services"
-  #+win32 (string-concat (getenv "WINDIR") "/system32/drivers/etc/services"))
+  #+win32 (string-concat (getenv "WINDIR") "/system32/drivers/etc/services")
+  #-(or unix win32)
+  (error "~S: only ~S and ~S are supported" '*services-file* :unix :win32))
 (defun service (&optional service-name protocol)
   (unless (boundp '*services*)
     (setq *services*
@@ -357,19 +366,18 @@
 
 ;;;--------------------------------------------------------------------------
 #+unix
-(defun make-xterm-io-stream (&key (title "CLISP I/O"))
-  (let* ((tmps (mkstemp "/tmp/clisp-x-io-XXXXXX"))
-         (pipe (namestring tmps))
+(defun make-xterm-io-stream (&key (title "CLISP I/O") (xterm "xterm"))
+  (let* ((tmpdir (mkdtemp "/tmp/clisp-x-io-XXXXXX/"))
+         (pipe (namestring (merge-pathnames "pipe" tmpdir)))
          xio
          (clos::*enable-clos-warnings* nil))
-    (close tmps) (delete-file tmps)
-    (mknod tmps :FIFO :RWXU)
+    (mknod pipe :FIFO :RWXU)
     (setq title (string title))
     ;; - tty tells us what device to use for IO
     ;; - cat holds xterm from quitting and prints the good-bye message
     ;;   (actually, the xterm window disappears before you can read it...)
-    (shell (format nil "xterm -n ~s -T ~s -e 'tty >> ~a; cat ~a' &"
-                   title title pipe pipe))
+    (shell (format nil "~a -n ~s -T ~s -e 'tty >> ~a; cat ~a' &"
+                   xterm title title pipe pipe))
     (with-open-file (s pipe :direction :input)
       (setq xio (open (read-line s) :direction :io)))
     ;; GC uses the internal non-generic builtin_stream_close() so
@@ -383,13 +391,18 @@
       (with-open-file (s pipe :direction :output)
         (write-line (SYS::TEXT "Bye.") s))
       (delete-file pipe)
+      (delete-directory tmpdir)
       (let ((clos::*enable-clos-warnings* nil))
         (remove-method #'close (find-method #'close '(:after) `((eql ,xio))))))
     xio))
-
 ;;;--------------------------------------------------------------------------
-#+FFI (export '(fopen fdopen fclose fflush ; fgetc fputc ungetc
-                clearerr feof ferror fileno))
+#+FFI (progn
+(export '(fopen fdopen freopen fclose fflush ; fgetc fputc ungetc
+          clearerr feof ferror fileno stdin stdout stderr))
+(defconstant stdin (%stdio 0))
+(defconstant stdout (%stdio 1))
+(defconstant stderr (%stdio 2))
+)
 ;;;--------------------------------------------------------------------------
 (defun ffs (n) (integer-length (logand n (- n))))
 ;; http://www.opengroup.org/onlinepubs/009695399/functions/ffs.html
@@ -397,6 +410,11 @@
 ;; (ffi:def-call-out ffs (:name "ffs") (:arguments (i ffi:int))
 ;;   (:return-type ffi:int) (:language :stdc) (:library :default))
 ;; and it also supports bignums.
+(unless (fboundp 'os:tgamma)
+  ;; https://sourceforge.net/tracker/?func=detail&atid=101355&aid=1966375&group_id=1355
+  (defun os:tgamma (x)
+    (multiple-value-bind (lg sg) (os:lgamma x)
+      (* (or sg 1) (exp lg)))))
 ;;;--------------------------------------------------------------------------
 (setf (package-lock "EXT") nil)
 (use-package '("POSIX") "EXT")
